package com.koobietech.eas.common.utils;

import com.koobietech.eas.dao.constant.UserType;
import com.koobietech.eas.dao.login.pojo.UserDetail;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.Objects;

/**
 * @author lc
 */
public class SecurityUtils {

    /**
     * 取登录名
     * @return
     */
    public static String getLoginUserName() {
        UserDetail principal = getPrincipal();
        if (Objects.nonNull(principal)) {
            return principal.getUsername();
        }
        return "anonymousUser";
    }

    /**
     * 取用户ID
     * @return
     */
    public static Long getLoginUid() {
        UserDetail principal = getPrincipal();
        if (Objects.nonNull(principal)) {
            if ( !isStudentUser() ) {
                return principal.getId();
            }
        }
        return 0L;
    }

    /**
     * 判断是否是学员
     * @return
     */
    public static boolean isStudentUser() {
        boolean isStudent = false;
        UserDetail principal = getPrincipal();
        if (Objects.nonNull(principal)) {
            if ( principal.getUserType().equals(UserType.MEMBER) ) {
                isStudent = true;
            }
        }
        return isStudent;
    }

    /**
     * 取学员登录ID
     * @return
     */
    public static Long getLoginSid() {
        UserDetail principal = getPrincipal();
        if (Objects.nonNull(principal)) {
            if ( isStudentUser() ) {
                return principal.getId();
            }
        }
        return 0L;
    }

    /**
     * 是否登录
     * @return
     */
    public static boolean isLogged() {
        UserDetail principal = getPrincipal();
        return Objects.nonNull(principal);
    }

    private static UserDetail getPrincipal() {
        Object principal = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        if (Objects.nonNull(principal)) {
            if (principal instanceof UserDetail) {
                return (UserDetail) principal;
            } else {
                return null;
            }
        }
        return null;
    }
}
